@@ -19,6 +19,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import print_function
+
 import os
 
 
@@ -28,7 +30,7 @@ import os
 # that this isn't the 0.15 release version while also allowing us to release
 # an 0.15.1 if need be.  Release versions should use 2-digit version numbers,
 # with 0.16 being the next release version in this example.
-version = "0.20"
+version = "0.22"
 version_info = tuple([int(x) for x in version.split(".")])
 
 
@@ -53,6 +55,6 @@ if os.environ.get("STORM_CEXTENSIONS") != "0":
     try:
         from storm import cextensions
         has_cextensions = True
-    except ImportError, e:
+    except ImportError as e:
         if "cextensions" not in str(e):
            raise
