@@ -25,6 +25,16 @@ This is the common code for database support; specific databases are
 supported in modules in L{storm.databases}.
 """
 
+from __future__ import print_function
+
+try:
+    from collections.abc import Callable
+except ImportError:
+    from collections import Callable
+from functools import wraps
+
+import six
+
 from storm.expr import Expr, State, compile
 # Circular import: imported at the end of the module.
 # from storm.tracer import trace
@@ -32,7 +42,7 @@ from storm.variables import Variable
 from storm.xid import Xid
 from storm.exceptions import (
     ClosedError, ConnectionBlockedError, DatabaseError, DisconnectionError,
-    Error, ProgrammingError)
+    Error, ProgrammingError, wrap_exceptions)
 from storm.uri import URI
 import storm
 
@@ -161,6 +171,78 @@ class Result(object):
         return row
 
 
+class CursorWrapper(object):
+    """A DB-API cursor, wrapping exceptions as StormError instances."""
+
+    def __init__(self, cursor, database):
+        super(CursorWrapper, self).__setattr__('_cursor', cursor)
+        super(CursorWrapper, self).__setattr__('_database', database)
+
+    def __getattr__(self, name):
+        attr = getattr(self._cursor, name)
+        if isinstance(attr, Callable):
+            @wraps(attr)
+            def wrapper(*args, **kwargs):
+                with wrap_exceptions(self._database):
+                    return attr(*args, **kwargs)
+
+            return wrapper
+        else:
+            return attr
+
+    def __setattr__(self, name, value):
+        return setattr(self._cursor, name, value)
+
+    def __iter__(self):
+        with wrap_exceptions(self._database):
+            for item in self._cursor:
+                yield item
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type_, value, tb):
+        with wrap_exceptions(self._database):
+            self.close()
+
+
+class ConnectionWrapper(object):
+    """A DB-API connection, wrapping exceptions as StormError instances."""
+
+    def __init__(self, connection, database):
+        self.__dict__['_connection'] = connection
+        self.__dict__['_database'] = database
+
+    def __getattr__(self, name):
+        attr = getattr(self._connection, name)
+        if isinstance(attr, Callable):
+            @wraps(attr)
+            def wrapper(*args, **kwargs):
+                with wrap_exceptions(self._database):
+                    return attr(*args, **kwargs)
+
+            return wrapper
+        else:
+            return attr
+
+    def __setattr__(self, name, value):
+        return setattr(self._connection, name, value)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type_, value, tb):
+        with wrap_exceptions(self._database):
+            if type_ is None and value is None and tb is None:
+                self.commit()
+            else:
+                self.rollback()
+
+    def cursor(self):
+        with wrap_exceptions(self._database):
+            return CursorWrapper(self._connection.cursor(), self._database)
+
+
 class Connection(object):
     """A connection to a database.
 
@@ -318,7 +400,7 @@ class Connection(object):
                         self._raw_connection.tpc_rollback()
                     else:
                         self._raw_connection.rollback()
-                except Error, exc:
+                except Error as exc:
                     if self.is_disconnection_error(exc):
                         self._raw_connection = None
                         self._state = STATE_RECONNECT
@@ -386,7 +468,7 @@ class Connection(object):
         """Complete the statement execution, along with result reports."""
         try:
             self._check_disconnect(raw_cursor.execute, *args)
-        except Exception, error:
+        except Exception as error:
             self._check_disconnect(
                 trace, "connection_raw_execute_error", self, raw_cursor,
                 statement, params or (), error)
@@ -402,7 +484,7 @@ class Connection(object):
             self._check_disconnect(
                 trace, "connection_raw_execute", self, raw_cursor,
                 statement, params or ())
-        except Exception, error:
+        except Exception as error:
             self._check_disconnect(
                 trace, "connection_raw_execute_error", self, raw_cursor,
                 statement, params or (), error)
@@ -423,7 +505,7 @@ class Connection(object):
         elif self._state == STATE_RECONNECT:
             try:
                 self._raw_connection = self._database.raw_connect()
-            except DatabaseError, exc:
+            except DatabaseError as exc:
                 self._state = STATE_DISCONNECTED
                 self._raw_connection = None
                 raise DisconnectionError(str(exc))
@@ -452,7 +534,7 @@ class Connection(object):
             'extra_disconnection_errors', ())
         try:
             return function(*args, **kwargs)
-        except Exception, exc:
+        except Exception as exc:
             if self.is_disconnection_error(exc, extra_disconnection_errors):
                 self._state = STATE_DISCONNECTED
                 self._raw_connection = None
@@ -479,6 +561,14 @@ class Database(object):
 
     connection_factory = Connection
 
+    def __init__(self, uri=None):
+        self._uri = uri
+        self._exception_types = {}
+
+    def get_uri(self):
+        """Return the URI object this database was created with."""
+        return self._uri
+
     def connect(self, event=None):
         """Create a connection to the database.
 
@@ -503,6 +593,58 @@ class Database(object):
         """
         raise NotImplementedError
 
+    @property
+    def _exception_module(self):
+        """The module where appropriate DB-API exception types are defined.
+
+        Subclasses should set this if they support re-raising DB-API
+        exceptions as StormError instances.
+        """
+        return None
+
+    def _make_combined_exception_type(self, wrapper_type, dbapi_type):
+        """Make a combined exception based on both DB-API and Storm.
+
+        Storm historically defined its own exception types as ABCs and
+        registered the DB-API exception types as virtual subclasses.
+        However, this doesn't work properly in Python 3
+        (https://bugs.python.org/issue12029).  Instead, we create and cache
+        subclass-specific exception types that inherit from both StormError
+        and the DB-API exception type, allowing code that catches either
+        StormError (or subclasses) or the specific DB-API exceptions to keep
+        working.
+
+        @type wrapper_type: L{type}
+        @param wrapper_type: The type of the wrapper exception to create; a
+            subclass of L{StormError}.
+        @type dbapi_type: L{type}
+        @param dbapi_type: The type of the DB-API exception.
+
+        @return: The combined exception type.
+        """
+        if dbapi_type.__name__ not in self._exception_types:
+            self._exception_types[dbapi_type.__name__] = type(
+                dbapi_type.__name__, (dbapi_type, wrapper_type), {})
+        return self._exception_types[dbapi_type.__name__]
+
+    def _wrap_exception(self, wrapper_type, exception):
+        """Wrap a DB-API exception as a StormError instance.
+
+        This constructs a wrapper exception with the same C{args} as the
+        DB-API exception.  Subclasses may override this to set additional
+        attributes on the wrapper exception.
+
+        @type wrapper_type: L{type}
+        @param wrapper_type: The type of the wrapper exception to create; a
+            subclass of L{StormError}.
+        @type exception: L{Exception}
+        @param exception: The DB-API exception to wrap.
+
+        @return: The wrapped exception; an instance of L{StormError}.
+        """
+        return self._make_combined_exception_type(
+            wrapper_type, exception.__class__)(*exception.args)
+
 
 def convert_param_marks(statement, from_param_mark, to_param_mark):
     # TODO: Add support for $foo$bar$foo$ literals.
@@ -537,7 +679,7 @@ def create_database(uri):
         - "anything:..." Where 'anything' has previously been registered
           with L{register_scheme}.
     """
-    if isinstance(uri, basestring):
+    if isinstance(uri, six.string_types):
         uri = URI(uri)
     if uri.scheme in _database_schemes:
         factory = _database_schemes[uri.scheme]
