@@ -23,9 +23,13 @@
 L{SQLObjectBase} is the central point of compatibility.
 """
 
+from __future__ import print_function
+
 import re
 import warnings
 
+import six
+
 from storm.properties import (
     RawStr, Int, Bool, Float, DateTime, Date, TimeDelta)
 from storm.references import Reference, ReferenceSet
@@ -162,7 +166,9 @@ class SQLObjectMeta(PropertyPublisherMeta):
         dict["__storm_table__"] = table_name
 
         attr_to_prop = {}
-        for attr, prop in dict.items():
+        for attr, prop in list(six.iteritems(dict)):
+            if attr == "__classcell__":  # Python >= 3.6
+                continue
             attr_to_prop[attr] = attr
             if isinstance(prop, ForeignKey):
                 db_name = prop.kwargs.get("dbName", attr)
@@ -205,7 +211,7 @@ class SQLObjectMeta(PropertyPublisherMeta):
 
 
         id_type = dict.setdefault("_idType", int)
-        id_cls = {int: Int, str: RawStr, unicode: AutoUnicode}[id_type]
+        id_cls = {int: Int, bytes: RawStr, six.text_type: AutoUnicode}[id_type]
         dict["id"] = id_cls(id_name, primary=True, default=AutoReload)
         attr_to_prop[id_name] = "id"
 
@@ -222,7 +228,7 @@ class SQLObjectMeta(PropertyPublisherMeta):
         property_registry.add_property(obj, getattr(obj, "id"),
                                        "<table %s>" % table_name)
 
-        for fake_name, real_name in attr_to_prop.items():
+        for fake_name, real_name in list(six.iteritems(attr_to_prop)):
             prop = getattr(obj, real_name)
             if fake_name != real_name:
                 property_registry.add_property(obj, prop, fake_name)
@@ -259,7 +265,7 @@ class BoundDotQ(object):
             return getattr(self._cls, attr)
 
 
-class SQLObjectBase(Storm):
+class SQLObjectBase(six.with_metaclass(SQLObjectMeta, Storm)):
     """The root class of all SQLObject-emulating classes in your application.
 
     The general strategy for using Storm's SQLObject emulation layer
@@ -269,7 +275,6 @@ class SQLObjectBase(Storm):
     even be implemented as returning a global L{Store} instance. Then
     all database classes should subclass that class.
     """
-    __metaclass__ = SQLObjectMeta
 
     q = DotQ()
     _SO_creating = False
@@ -296,7 +301,7 @@ class SQLObjectBase(Storm):
         self._init(None)
 
     def set(self, **kwargs):
-        for attr, value in kwargs.iteritems():
+        for attr, value in six.iteritems(kwargs):
             setattr(self, attr, value)
 
     def destroySelf(self):
@@ -329,7 +334,7 @@ class SQLObjectBase(Storm):
         if not isinstance(orderBy, (tuple, list)):
             orderBy = (orderBy,)
         for item in orderBy:
-            if isinstance(item, basestring):
+            if isinstance(item, six.string_types):
                 desc = item.startswith("-")
                 if desc:
                     item = item[1:]
@@ -404,7 +409,7 @@ class SQLObjectResultSet(object):
 
     def _copy(self, **kwargs):
         copy = self.__class__(self._cls, **kwargs)
-        for name, value in self.__dict__.iteritems():
+        for name, value in six.iteritems(self.__dict__):
             if name[1:] not in kwargs and name != "_finished_result_set":
                 setattr(copy, name, value)
         return copy
@@ -548,15 +553,18 @@ class SQLObjectResultSet(object):
         result_set = self._without_prejoins()._result_set
         return item in result_set
 
-    def __nonzero__(self):
+    def __bool__(self):
         """Return C{True} if this result set contains any results.
 
         @note: This method is provided for compatibility with SQL Object.  For
             new code, prefer L{is_empty}.  It's compatible with L{ResultSet}
-            which doesn't have a C{__nonzero__} implementation.
+            which doesn't have a C{__bool__} implementation.
         """
         return not self.is_empty()
 
+    if six.PY2:
+        __nonzero__ = __bool__
+
     def is_empty(self):
         """Return C{True} if this result set doesn't contain any results."""
         result_set = self._without_prejoins()._result_set
@@ -603,7 +611,7 @@ class SQLObjectResultSet(object):
         return self._copy(prejoinClauseTables=prejoinClauseTables)
 
     def sum(self, attribute):
-        if isinstance(attribute, basestring):
+        if isinstance(attribute, six.string_types):
             attribute = SQL(attribute)
         result_set = self._without_prejoins()._result_set
         return result_set.sum(attribute)
@@ -675,13 +683,21 @@ class PropertyAdapter(object):
 
 
 class AutoUnicodeVariable(Variable):
-    """Unlike UnicodeVariable, this will try to convert str to unicode."""
+    """A more relaxed version of UnicodeVariable that accepts native strings.
+
+    On Python 2, this will try to convert bytes to text, to make it easier
+    to port code from SQLObject that expects to be able to set this variable
+    to a native string.
+
+    On Python 3, this behaves the same way as UnicodeVariable and only
+    accepts text, since native strings are already Unicode.
+    """
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, basestring):
-            raise TypeError("Expected basestring, found %s" % repr(type(value)))
-        return unicode(value)
+        if not isinstance(value, six.string_types):
+            raise TypeError("Expected a string type, found %r" % type(value))
+        return six.text_type(value)
 
 class AutoUnicode(SimpleProperty):
     variable_class = AutoUnicodeVariable
