@@ -24,10 +24,14 @@
 This module contains the highest-level ORM interface in Storm.
 """
 
+from __future__ import print_function
+
 from copy import copy
 from weakref import WeakValueDictionary
 from operator import itemgetter
 
+import six
+
 from storm.info import get_cls_info, get_obj_info, set_obj_info
 from storm.variables import Variable, LazyValue
 from storm.expr import (
@@ -37,6 +41,7 @@ from storm.expr import (
 from storm.exceptions import (
     WrongStoreError, NotFlushedError, OrderLoopError, UnorderedError,
     NotOneError, FeatureError, CompileError, LostObjectError, ClassInfoError)
+from storm.properties import PropertyColumn
 from storm import Undef
 from storm.cache import Cache
 from storm.event import EventSystem
@@ -234,7 +239,7 @@ class Store(object):
         required. An example follows::
 
             join = LeftJoin(Person, Person.id == Company.person_id)
-            print list(store.using(Company, join).find((Company, Person)))
+            print(list(store.using(Company, join).find((Company, Person))))
 
         The previous code snippet will produce an SQL statement
         somewhat similar to this, depending on your backend::
@@ -469,7 +474,7 @@ class Store(object):
         self._dirty = flushing
 
         predecessors = {}
-        for (before_info, after_info), n in self._order.iteritems():
+        for (before_info, after_info), n in six.iteritems(self._order):
             if n > 0:
                 before_set = predecessors.get(after_info)
                 if before_set is None:
@@ -848,7 +853,7 @@ class Store(object):
             del obj_info["primary_vars"]
 
     def _iter_alive(self):
-        return self._alive.values()
+        return list(six.itervalues(self._alive))
 
     def _enable_change_notification(self, obj_info):
         obj_info.event.emit("start-tracking-changes", self._event)
@@ -1002,7 +1007,7 @@ class ResultSet(object):
             L{ResultSet} will be returned appropriately modified with
             C{OFFSET} and C{LIMIT} clauses.
         """
-        if isinstance(index, (int, long)):
+        if isinstance(index, six.integer_types):
             if index == 0:
                 result_set = self
             else:
@@ -1401,7 +1406,7 @@ class ResultSet(object):
                     for column in changes:
                         obj_info.variables[column].set(AutoReload)
         else:
-            changes = changes.items()
+            changes = list(six.iteritems(changes))
             for obj in cached:
                 for column, value in changes:
                     variables = get_obj_info(obj).variables
@@ -1706,7 +1711,12 @@ class FindSpec(object):
         return columns, default_tables
 
     def is_compatible(self, find_spec):
-        """Return True if this FindSpec is compatible with a second one."""
+        """Return True if this FindSpec is compatible with a second one.
+
+        Two FindSpecs are considered compatible if either the find specs
+        are identical (i.e. specifies the same classes and columns) or
+        the find spec columns are of the same type.
+        """
         if self.is_tuple != find_spec.is_tuple:
             return False
         if len(self._cls_spec_info) != len(find_spec._cls_spec_info):
@@ -1715,7 +1725,18 @@ class FindSpec(object):
             self._cls_spec_info, find_spec._cls_spec_info):
             if is_expr1 != is_expr2:
                 return False
-            if info1 is not info2:
+            # If both infos are PropertyColumns, check whether they are
+            # of the same type. Ideally we should check that the types as
+            # defined in the database are the same, but checking the
+            # variable class is easier and will work most of the time.
+            if isinstance(info1, PropertyColumn):
+                if not isinstance(info2, PropertyColumn):
+                    return False
+                variable_class1 = info1.variable_factory().__class__
+                variable_class2 = info2.variable_factory().__class__
+                if variable_class1 is not variable_class2:
+                    return False
+            elif info1 is not info2:
                 return False
         return True
 
