@@ -18,14 +18,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import print_function
+
 from datetime import datetime, date, time, timedelta
 from decimal import Decimal
-import cPickle as pickle
+from functools import partial
 import re
-try:
-    import uuid
-except ImportError:
-    uuid = None
+import uuid
+import weakref
+
+import six
+from six.moves import cPickle as pickle
 
 from storm.compat import json
 from storm.exceptions import NoneError
@@ -54,6 +57,12 @@ __all__ = [
 ]
 
 
+if six.PY3:
+    _buffer_type = memoryview
+else:
+    _buffer_type = buffer
+
+
 class LazyValue(object):
     """Marker to be used as a base class on lazily evaluated values."""
     __slots__ = ()
@@ -74,22 +83,7 @@ def raise_none_error(column):
         raise NoneError("None isn't acceptable as a value for %s" % name)
 
 
-def VariableFactory(cls, **old_kwargs):
-    """Build cls with kwargs of constructor updated by kwargs of call.
-
-    This is really an implementation of partial/curry functions, and
-    is replaced by 'partial' when 2.5+ is in use.
-    """
-    def variable_factory(**new_kwargs):
-        kwargs = old_kwargs.copy()
-        kwargs.update(new_kwargs)
-        return cls(**kwargs)
-    return variable_factory
-
-try:
-    from functools import partial as VariableFactory
-except ImportError:
-    pass
+VariableFactory = partial
 
 
 class Variable(object):
@@ -153,7 +147,7 @@ class Variable(object):
             self._validator_object_factory = validator_object_factory
             self._validator_attribute = validator_attribute
         self.column = column
-        self.event = event
+        self.event = weakref.proxy(event) if event is not None else None
 
     def get_lazy(self, default=None):
         """Get the current L{LazyValue} without resolving its value.
@@ -329,7 +323,7 @@ class BoolVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, (int, long, float, Decimal)):
+        if not isinstance(value, (six.integer_types, float, Decimal)):
             raise TypeError("Expected bool, found %r: %r"
                             % (type(value), value))
         return bool(value)
@@ -339,7 +333,7 @@ class IntVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, (int, long, float, Decimal)):
+        if not isinstance(value, (six.integer_types, float, Decimal)):
             raise TypeError("Expected int, found %r: %r"
                             % (type(value), value))
         return int(value)
@@ -349,7 +343,7 @@ class FloatVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, (int, long, float, Decimal)):
+        if not isinstance(value, (six.integer_types, float, Decimal)):
             raise TypeError("Expected float, found %r: %r"
                             % (type(value), value))
         return float(value)
@@ -360,8 +354,8 @@ class DecimalVariable(Variable):
 
     @staticmethod
     def parse_set(value, from_db):
-        if (from_db and isinstance(value, basestring) or
-            isinstance(value, (int, long))):
+        if (from_db and isinstance(value, six.string_types) or
+            isinstance(value, six.integer_types)):
             value = Decimal(value)
         elif not isinstance(value, Decimal):
             raise TypeError("Expected Decimal, found %r: %r"
@@ -371,7 +365,7 @@ class DecimalVariable(Variable):
     @staticmethod
     def parse_get(value, to_db):
         if to_db:
-            return unicode(value)
+            return six.text_type(value)
         return value
 
 
@@ -379,10 +373,10 @@ class RawStrVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if isinstance(value, buffer):
-            value = str(value)
-        elif not isinstance(value, str):
-            raise TypeError("Expected str, found %r: %r"
+        if isinstance(value, _buffer_type):
+            value = bytes(value)
+        elif not isinstance(value, bytes):
+            raise TypeError("Expected bytes, found %r: %r"
                             % (type(value), value))
         return value
 
@@ -391,8 +385,8 @@ class UnicodeVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        if not isinstance(value, unicode):
-            raise TypeError("Expected unicode, found %r: %r"
+        if not isinstance(value, six.text_type):
+            raise TypeError("Expected text, found %r: %r"
                             % (type(value), value))
         return value
 
@@ -408,7 +402,7 @@ class DateTimeVariable(Variable):
         if from_db:
             if isinstance(value, datetime):
                 pass
-            elif isinstance(value, (str, unicode)):
+            elif isinstance(value, six.string_types):
                 if " " not in value:
                     raise ValueError("Unknown date/time format: %r" % value)
                 date_str, time_str = value.split(" ")
@@ -422,11 +416,22 @@ class DateTimeVariable(Variable):
                 else:
                     value = value.astimezone(self._tzinfo)
         else:
-            if type(value) in (int, long, float):
+            if type(value) in six.integer_types + (float, ):
                 value = datetime.utcfromtimestamp(value)
             elif not isinstance(value, datetime):
                 raise TypeError("Expected datetime, found %s" % repr(value))
             if self._tzinfo is not None:
+                # Python 3.6 gained support for calling the astimezone
+                # method on naive datetime objects, presuming them to
+                # represent system local time.  This is probably
+                # inappropriate for most uses of Storm, since depending on
+                # what the system local time happens to be is usually a
+                # mistake, so forbid this explicitly for now; we can always
+                # open it up later if there's a good reason.
+                if (value.tzinfo is None or
+                        value.tzinfo.utcoffset(value) is None):
+                    raise ValueError(
+                        "Expected aware datetime, found naive: %r" % value)
                 value = value.astimezone(self._tzinfo)
         return value
 
@@ -442,7 +447,7 @@ class DateVariable(Variable):
                 return value.date()
             if isinstance(value, date):
                 return value
-            if not isinstance(value, (str, unicode)):
+            if not isinstance(value, six.string_types):
                 raise TypeError("Expected date, found %s" % repr(value))
             if " " in value:
                 value, time_str = value.split(" ")
@@ -465,7 +470,7 @@ class TimeVariable(Variable):
                 return None
             if isinstance(value, time):
                 return value
-            if not isinstance(value, (str, unicode)):
+            if not isinstance(value, six.string_types):
                 raise TypeError("Expected time, found %s" % repr(value))
             if " " in value:
                 date_str, value = value.split(" ")
@@ -488,7 +493,7 @@ class TimeDeltaVariable(Variable):
                 return None
             if isinstance(value, timedelta):
                 return value
-            if not isinstance(value, (str, unicode)):
+            if not isinstance(value, six.string_types):
                 raise TypeError("Expected timedelta, found %s" % repr(value))
             return _parse_interval(value)
         else:
@@ -501,8 +506,7 @@ class UUIDVariable(Variable):
     __slots__ = ()
 
     def parse_set(self, value, from_db):
-        assert uuid is not None, "The uuid module was not found."
-        if from_db and isinstance(value, basestring):
+        if from_db and isinstance(value, six.string_types):
             value = uuid.UUID(value)
         elif not isinstance(value, uuid.UUID):
             raise TypeError("Expected UUID, found %r: %r"
@@ -511,7 +515,7 @@ class UUIDVariable(Variable):
 
     def parse_get(self, value, to_db):
         if to_db:
-            return unicode(value)
+            return six.text_type(value)
         return value
 
 
@@ -557,7 +561,7 @@ class MutableValueVariable(Variable):
             self.event.hook("object-deleted", self._detect_changes_and_stop)
 
     def _start_tracking(self, obj_info, event_system):
-        self._event_system = event_system
+        self._event_system = weakref.proxy(event_system)
         self.event.hook("stop-tracking-changes", self._stop_tracking)
 
     def _stop_tracking(self, obj_info, event_system):
@@ -594,8 +598,8 @@ class EncodedValueVariable(MutableValueVariable):
 
     def parse_set(self, value, from_db):
         if from_db:
-            if isinstance(value, buffer):
-                value = str(value)
+            if isinstance(value, _buffer_type):
+                value = bytes(value)
             return self._loads(value)
         else:
             return value
@@ -627,13 +631,8 @@ class JSONVariable(EncodedValueVariable):
 
     __slots__ = ()
 
-    def __init__(self, *args, **kwargs):
-        assert json is not None, (
-            "Neither the json nor the simplejson module was found.")
-        super(JSONVariable, self).__init__(*args, **kwargs)
-
     def _loads(self, value):
-        if not isinstance(value, unicode):
+        if not isinstance(value, six.text_type):
             raise TypeError(
                 "Cannot safely assume encoding of byte string %r." % value)
         return json.loads(value)
@@ -641,9 +640,9 @@ class JSONVariable(EncodedValueVariable):
     def _dumps(self, value):
         # http://www.ietf.org/rfc/rfc4627.txt states that JSON is text-based
         # and so we treat it as such here. In other words, this method returns
-        # unicode and never str.
+        # Unicode text and never bytes.
         dump = json.dumps(value, ensure_ascii=False)
-        if not isinstance(dump, unicode):
+        if not isinstance(dump, six.text_type):
             # json.dumps() does not always return unicode. See
             # http://code.google.com/p/simplejson/issues/detail?id=40 for one
             # of many discussions of str/unicode handling in simplejson.
